@@ -123,6 +123,10 @@ between versions in an incompatible way!
 We (try to) use US English for identifiers while using GB English for the
 documentation.  This is intended and not a bug!
 
+Some CPAN smoke testers fail some of the test scripts (notably #11, #14, #20
+and #41) when the size of their virtual terminal is too small (height less
+than 12 lines)
+
 =cut
 
 #########################################################################
@@ -135,7 +139,7 @@ use warnings 'once';
 
 use Carp;			# may only be used in import!
 
-our $VERSION = "0.38";
+our $VERSION = "0.39";
 
 BEGIN  {  require UI::Various::core;  }
 
@@ -400,7 +404,6 @@ sub using()
 
 1;
 
-__END__
 #########################################################################
 #########################################################################
 
